@@ -5,8 +5,8 @@
 ##======================================
 #author: Sebastian Kreutzer
 #organisation: JLU Giessen
-#vers.: 0.2.2
-#date: 16/05/2012
+#vers.: 0.2.5
+#date: 19/07/2012
 ##======================================
 ##script analysis OSL data from a SAR measurement
 ##  --Input: RisoeBINfile object
@@ -24,6 +24,7 @@ Analyse_SAR.OSLdata<-function(input.data,
                          run,
                          set,
                          info.measurement="unkown measurement",
+                         log="",
                          output.plot=FALSE,
                          cex.global=1                         
                          ){
@@ -34,7 +35,7 @@ Analyse_SAR.OSLdata<-function(input.data,
 
 ##set colors gallery to provide more colors
     col<-unlist(colors())
-    col<-col[c(261,552,51,62,76,151,451,474,654)]
+    col<-col[c(261,552,51,62,76,151,451,474,654,657,100,513,23,612,129,27,551,393)]
     
 ##=================================================================================================##
 ##ERROR HANDLING
@@ -50,11 +51,13 @@ Analyse_SAR.OSLdata<-function(input.data,
   if(missing(position)==TRUE){position<-min(sample.data@METADATA[,"POSITION"]):max(sample.data@METADATA[,"POSITION"])}
   if(missing(run)==TRUE){run<-min(sample.data@METADATA[,"RUN"]):max(sample.data@METADATA[,"RUN"])}
   if(missing(set)==TRUE){set<-min(sample.data@METADATA[,"SET"]):max(sample.data@METADATA[,"SET"])}  
-  
+ 
+    
 ##=================================================================================================##
-##CALCULATIONS
+##CALCULATIONS 
 ##=================================================================================================##
    
+    
 ##loop over all positions
 for (i in position){
     
@@ -145,17 +148,17 @@ if(length(which(sample.data@METADATA["POSITION"]==i))>0){
                     Repeated=as.logical(temp.DoseName[,"Repeated"]))
     LnLxTnTx<-cbind(temp.LnLxTnTx,LnLxTnTx)
     LnLxTnTx[,"Name"]<-as.character(LnLxTnTx[,"Name"])
-        
+  
     ##(6) Calculate Recyling Ratio and Recuperation Rate 
       
       ##(6.1)
       ##Calculate Recycling Ratio 
-      
+   
       if(length(LnLxTnTx[LnLxTnTx[,"Repeated"]==TRUE,"Repeated"])>0){
                      
               ##identify repeated doses
               temp.Repeated<-LnLxTnTx[LnLxTnTx[,"Repeated"]==TRUE,c("Name","Dose","LxTx")]
-             
+            
               ##find concering previous dose for the repeated dose
               temp.Previous<-t(sapply(1:length(temp.Repeated[,1]),function(x){
                   LnLxTnTx[LnLxTnTx[,"Dose"]==temp.Repeated[x,"Dose"] & 
@@ -164,14 +167,13 @@ if(length(which(sample.data@METADATA["POSITION"]==i))>0){
              
               ##convert to data.frame
               temp.Previous<-as.data.frame(temp.Previous)
-          
+            
               ##set column names
               temp.ColNames<-sapply(1:length(temp.Repeated[,1]),function(x){
-                paste(temp.Previous[temp.Previous[x,"Dose"]==temp.Repeated[x,"Dose"],"Name"],"/",
+                     paste(temp.Previous[temp.Previous[,"Dose"]==temp.Repeated[x,"Dose"],"Name"],"/",
                                     temp.Repeated[x,"Name"],sep="")
                       })
-          
-                                 
+                                            
               ##Calculate Recycling Ratio
               RecyclingRatio<-as.numeric(temp.Previous[,"LxTx"])/as.numeric(temp.Repeated[,"LxTx"])
               
@@ -183,12 +185,12 @@ if(length(which(sample.data@METADATA["POSITION"]==i))>0){
     
       ##(6.2)
       ##Recuperation Rate
-
-      if(LnLxTnTx[LnLxTnTx[,"Name"]=="R0","Name"]=="R0"){
+  
+      if("R0" %in% LnLxTnTx[,"Name"]==TRUE){
        Recuperation<-round(LnLxTnTx[LnLxTnTx[,"Name"]=="R0","LxTx"]/LnLxTnTx[LnLxTnTx[,"Name"]=="Natural","LxTx"],digits=4)
       }else{Recuperation<-NA}
     
-    
+  
       ##(6.3) IRSL 
       ##Print IRSL Curves if IRSL curve is set
       sample.data@METADATA[,"SEL"]<-FALSE
@@ -228,10 +230,16 @@ if(length(which(sample.data@METADATA["POSITION"]==i))>0){
 if(output.plot==TRUE){
  
  layout(matrix(c(1,2,1,2,3,4,3,5),4,2,byrow=TRUE))
-       
+     
+    ##warning if number of curves exceed colour values
+    if(length(col)<length(LnLx.curveID)){
+      cat("\n[Analyse_OSLCurves.R] Warning: To many curves! Only the first",length(col),"curves are plotted!")
+    }
+ 
     ##============================================================================================
     ##plot Ln,Lx Curves
 
+
       ##get maximum value of LnLx curves
       LnLx.curveMax<-max(unlist(sample.data@DATA[LnLx.curveID]))
 
@@ -240,19 +248,24 @@ if(output.plot==TRUE){
       NPOINTS<-sample.data@METADATA[sample.data@METADATA[,"ID"]==LnLx.curveID[1],"NPOINTS"]
          
       xaxt.values<-seq(HIGH/NPOINTS,HIGH,by=HIGH/NPOINTS)
-
+ 
       ##open plot area LnLx
       plot(NA,NA,
-          xlab="t [s]",
-          ylab=paste("OSL [cts/",HIGH/NPOINTS," s]",sep=""),
-          xlim=c(0,HIGH),
-          ylim=c(0,max(unlist(sample.data@DATA[LnLx.curveID]))),
-          main=expression(paste(L[n],",",L[x]," curves",sep="")) 
+          xlab=if(log=="x" | log=="xy"){"log t [s]"}else{"t [s]"},
+          ylab=if(log=="y" | log=="xy"){
+                 paste("log OSL [cts/",HIGH/NPOINTS," s]",sep="")
+               }else{
+                 paste("log OSL [cts/",HIGH/NPOINTS," s]",sep="")                 
+               },
+          xlim=c(HIGH/NPOINTS,HIGH),
+          ylim=c(1,max(unlist(sample.data@DATA[LnLx.curveID]))),
+          main=expression(paste(L[n],",",L[x]," curves",sep="")),
+          log=log
        )
           ##plot curves and get legend values
           sapply(1:length(LnLx.curveID),function(x){
                 yaxt.values<-unlist(sample.data@DATA[LnLx.curveID[x]])
-                lines(xaxt.values,yaxt.values,col=col[x])              
+                lines(xaxt.values,yaxt.values,col=col[x])
                 })
           
           ##mark integration limits
@@ -264,23 +277,28 @@ if(output.plot==TRUE){
 
           ##plot legend
           legend("topright",as.character(LnLxTnTx$Name),lty=c(rep(1,length(LnLx.curveID))),
-                 cex=0.8*cex.global,col=col, bg="white")
+                 cex=0.8*cex.global,col=col, bg="gray")
      
           ##sample name
           mtext(side=3,sample.data@METADATA[sample.data@METADATA[,"ID"]==LnLx.curveID[1],"SAMPLE"],cex=0.7*cex.global)
       ##============================================================================================    
       ##open plot area TnTx
       plot(NA,NA,
-          xlab="t [s]",
-          ylab=paste("OSL [cts/",HIGH/NPOINTS," s]",sep=""),
-          xlim=c(0,HIGH),
-          ylim=c(0,max(unlist(sample.data@DATA[TnTx.curveID]))),
-          main=expression(paste(T[n],",",T[x]," curves",sep="")) 
+          xlab=if(log=="x" | log=="xy"){"log t [s]"}else{"t [s]"},
+          ylab=if(log=="y" | log=="xy"){
+             paste("log OSL [cts/",HIGH/NPOINTS," s]",sep="")
+           }else{
+             paste("log OSL [cts/",HIGH/NPOINTS," s]",sep="")                 
+           },
+          xlim=c(HIGH/NPOINTS,HIGH),
+          ylim=c(1,max(unlist(sample.data@DATA[TnTx.curveID]))),
+          main=expression(paste(T[n],",",T[x]," curves",sep="")),
+          log=log
        )
           ##plot curves and get legend values
           sapply(1:length(TnTx.curveID),function(x){
                 yaxt.values<-unlist(sample.data@DATA[TnTx.curveID[x]])
-                lines(xaxt.values,yaxt.values,col=col[x])               
+                lines(xaxt.values,yaxt.values,col=col[x])   
                 })
           
           ##mark integration limits
@@ -292,7 +310,7 @@ if(output.plot==TRUE){
 
           ##plot legend
           legend("topright",as.character(LnLxTnTx$Name),lty=c(rep(1,length(TnTx.curveID))),
-                 cex=0.8*cex.global,col=col, bg="white")
+                 cex=0.8*cex.global,col=col, bg="gray")
     
          ##sample name
          mtext(side=3,sample.data@METADATA[sample.data@METADATA[,"ID"]==LnLx.curveID[1],"SAMPLE"],cex=0.7*cex.global)
@@ -332,11 +350,16 @@ if(output.plot==TRUE){
         ##open plot TL curves
         plot(NA,NA,
              xlab="T [deg. C]",
-             ylab=paste("TL [cts/",HIGH/NPOINTS," deg. C]",sep=""),
-             xlim=c(0,HIGH),
-             ylim=c(0,TL.curveMax),
+             ylab=if(log=="xy" | log=="y"){
+                    paste("log TL [cts/",HIGH/NPOINTS," deg. C]",sep="")
+                  }else{
+                    paste("TL [cts/",HIGH/NPOINTS," deg. C]",sep="") 
+                  },
+             xlim=c(HIGH/NPOINTS,HIGH),
+             ylim=c(1,TL.curveMax),
              main="Cutheat - TL curves",
-             sub=paste("(",RATE," K/s)",sep="")
+             sub=paste("(",RATE," K/s)",sep=""),
+             log=if(log=="y" | log=="xy"){"y"}else{""}
             ) 
     
         ##plot curves and get legend values
